/* ----------------------------------------------------------------------- 
 *	Copylight(C) 2018 Kanazawa-soft-design,LLC.
 * ----------------------------------------------------------------------- */
/*!
 *	@file	FileDataView.java
 *
 *	@brief	ファイルデータのバイナリ表示ビュー (Java + Swing).
 *
 *	@author	金澤 宣明
 */
package sample.JavaSwing001;

import java.io.File;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;


@SuppressWarnings("serial")
public class FileDataView extends JPanel implements AdjustmentListener {

	private JScrollBar	scrollH;							//!< 水平スクロールバー.
	private JScrollBar	scrollV;							//!< 垂直スクロールバー.
	private String		fileName = "";						//!< ファイル名.
	private int			fileSize = 0;						//!< ファイルサイズ.
	private byte[]		fileData;							//!< ファイルのデータバッファ.
	private final int	MAX_FILE_SIZE = (1024 * 1024 * 10);	//!< 読み込むファイルの最大バイト数.



	/* ----------------------------------------------------------------------- */
	/**
	 *	コンストラクタ.
	 *
	 * @param	vscroll		垂直スクロールバー.
	 * @param	hscroll		水平スクロールバー.
	 */
	FileDataView( JScrollBar vscroll, JScrollBar hscroll ) {

		//! 背景色をセットする.
		setBackground( Color.WHITE );

		//! スクロールバーのオブジェクトを保存する.
		scrollV = vscroll;
		scrollH = hscroll;

		//! 垂直スクロールバーを初期化する.
		scrollV.setMinimum( 0 );
		scrollV.setMaximum( 0 );
		scrollV.setValue(   0 );

		//! 水平スクロールバーを初期化する.
		scrollH.setMinimum( 0 );
		scrollH.setMaximum( 0 );
		scrollH.setValue(   0 );

		//! スクロールバーのイベントを登録する.
		scrollV.addAdjustmentListener( this );
		scrollH.addAdjustmentListener( this );

		//! ホイールマウスのイベントを登録する.
		addMouseWheelListener( new MouseWheelListener() {
			@Override
			public void mouseWheelMoved( MouseWheelEvent e ) {
				onMouseWheel( e );
			}
		});

		//! ウインドウサイズが変更されるときのイベントを登録する.
		addComponentListener( new ComponentListener() {
			@Override
			public void componentHidden( ComponentEvent e ) {
			}

			@Override
			public void componentMoved( ComponentEvent e ) {
			}

			@Override
			public void componentResized( ComponentEvent e ) {
				onResized( e );
			}

			@Override
			public void componentShown( ComponentEvent e ) {
			}
		});
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイルを読み込んでバイナリ形式で表示する.
	 *
	 *	@param	path	バイナリ表示するファイルの絶対パス.
	 */
	public void showBinaryFile( String path ) {

		fileName = path;

		try {
			File file = new File( fileName );

			//! ファイルのバイト数を取得する.
			fileSize = (int)file.length();

			/*!
			 * ファイルサイズが読み込みサイズの上限を超える場合は
			 * サイズの上限までしか読み込まない.
			 */
			if( fileSize > MAX_FILE_SIZE ){
				fileSize = MAX_FILE_SIZE;
			}

			if( fileSize > 0 ){
				//! ファイルサイズ分のファイルの読み込み用バッファを確保する.
				fileData = new byte[fileSize];

				//! ファイルを読み込みモードでオープンする.
				FileInputStream		fs = new FileInputStream( fileName );
				BufferedInputStream	bs = new BufferedInputStream( fs );
				DataInputStream		ds = new DataInputStream( bs );

				//! ファイルを読み込み用バッファに1回で読み込む.
				int readByte = ds.read( fileData );

				//! ファイルをクローズする.
				ds.close();

				//! 垂直スクロールバーを初期化する.
				scrollV.setMinimum( 0 );
				scrollV.setMaximum( (readByte + 15) / 16 );
				scrollV.setValue( 0 );

				//! 水平スクロールバーを初期化する.
				scrollH.setMinimum( 0 );
				scrollH.setMaximum( 74 );
				scrollH.setValue( 0 );

				//! 画面に合わせてスクロールバーのつまみ幅を設定する.
				setScrollBar();

				//! バイナリ表示ビューを再描画する.
				repaint();
			}
		} catch( FileNotFoundException e ) {
			fileSize = 0;
		} catch( IOException e ) {
			fileSize = 0;
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	バイナリ表示ビューを描画する.
	 *
	 *	@param	g	グラフィック･コンテキスト.
	 */
	@Override
	public void paintComponent( Graphics g ) {

		super.paintComponent( g );

		Graphics2D g2d = (Graphics2D)g;

		//! 固定幅フォントを使用する.
//		Font font = new Font( "Monospaced"   , Font.PLAIN, 14 );
//		Font font = new Font( Font.MONOSPACED, Font.PLAIN, 14 );
		Font font = new Font( "ＭＳ ゴシック", Font.PLAIN, 14 );
		g2d.setFont( font );

		//! フォントサイズを取得する.
		FontMetrics fm = g2d.getFontMetrics();
		int fontW = fm.charWidth( '0' );
		int fontH = fm.getHeight();

		//! 表示領域のサイズを取得する.
//		int viewW = getWidth();					// not used.
		int viewH = getHeight();

		//! 描画開始座標を取得する.
		int y = fm.getAscent();
		int x = -1 * scrollH.getValue();

		//! スケールを描画する.
		drawScale( g2d, x, y, fontW, fontH );

		//! 描画開始Y座標を更新する.
		y += fontH;

		if( fileSize > 0 ){
			//! 1画面に描画可能な行数を計算する.
			int lines = (viewH + (fontH - 1)) / fontH;

			//! 描画する行のオフセット･アドレスを計算する.
			int offset = scrollV.getValue() * 16;

			//! ファイルデータをバイナリ形式で描画する.
			for( int i = 1; i < lines; i++ ){
				if( offset >= fileSize ){
					break;
				}
				//! 1行分のバイナリデータを描画する.
				drawBinaryData( g2d, x, y, fontW, fontH, i, offset );
				offset += 16;

				//! 描画開始Y座標を更新する.
				y += fontH;
			}
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	スケールを描画する.
	 *
	 *	@param	g2d		グラフィック･コンテキスト.
	 *	@param	x		描画開始X座標.
	 *	@param	y		描画開始Y座標.
	 *	@param	fontW	フォントの幅.
	 *	@param	fontH	フォントの高さ.
	 */
	private void drawScale( Graphics2D g2d, int x, int y, int fontW, int fontH ) {

		//! 表示領域のサイズを取得する.
		int w = getWidth();
		int h = getHeight();

		g2d.setColor( new Color(128, 128, 128) );
		g2d.fillRect( x * fontW, 0, w - (x * fontW), fontH );
		g2d.fillRect( x * fontW, 0, fontW * 9, h );

		//! バイナリ表示ビューのスケールを描画する.
		g2d.setColor( Color.white );
		String str1 = "ADDRESS:";
		g2d.drawString( str1, x * fontW, y );

		String str2 = "+0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F";
		g2d.drawString( str2, (x + 10) * fontW, y );

		String str3 = "ASCII";
		g2d.drawString( str3, (x + 58) * fontW, y );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	1行分のバイナリデータを描画する.
	 *
	 *	@param	g2d		グラフィック･コンテキスト.
	 *	@param	x		描画開始X座標.
	 *	@param	y		描画開始Y座標.
	 *	@param	fontW	フォントの幅.
	 *	@param	fontH	フォントの高さ.
	 *	@param	line	描画開始する行位置.
	 *	@param	offset	描画開始するバイナリデータのアドレス.
	 */
	private void drawBinaryData( Graphics2D g2d, int x, int y, int fontW, int fontH, int line, int offset ) {

		//! この行のデータの先頭アドレスを描画する.
		String adr = String.format( "%08X", offset );
		g2d.setColor( Color.white );
		g2d.drawString( adr, x * fontW, y );

		//! この行で描画するバイト数を求める.
		int cnt = ((offset + 16) > fileSize ? (fileSize - offset) : 16);

		//! 1行分の表示データを生成する.
		String data  = new String();
		String ascii = new String();
		for( int i = 0; i < cnt; i++ ){
			int c = fileData[offset + i] & 0xFF;

			data +=  String.format( "%02X", c );
			data += " ";

			if( 0x20 <= c && c < 0x80 ){
				ascii += (char)c;
			}else{
				ascii += '.';
			}
		}
		//! バイナリデータを描画する.
		g2d.setColor( Color.black );
		g2d.drawString( data, (x + 10) * fontW, y );

		//! バイナリデータに対応したACSII文字列を描画する.
		g2d.drawString( ascii, (x + 58) * fontW, y );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	スクロールバーを設定する.
	 *
	 */
	private void setScrollBar() {

		//! クライアント領域のサイズを取得する.
		int viewW = getWidth();
		int viewH = getHeight();

		//! グラフィックス･オブジェクトを取得する.
		Graphics2D g2d = (Graphics2D)this.getGraphics();

		//! 固定幅フォントを作成する.
		Font font = new Font( "ＭＳ ゴシック", Font.PLAIN, 14 );
		g2d.setFont( font );

		//! フォントサイズを取得する.
		FontMetrics fm = g2d.getFontMetrics();
		int fontW = fm.charWidth( '0' );
		int fontH = fm.getHeight();

		//! 1画面に表示可能な行数を計算する.
		int lines = (viewH + (fontH - 1)) / fontH;

		//! 1画面に表示可能な文字数を計算する.
		int colomuns = (viewW + (fontW - 1)) / fontW;

		//! スクロールバーのつまみ幅を画面に合わせて再設定する.
		scrollV.setVisibleAmount( lines    / 2 );
		scrollH.setVisibleAmount( colomuns / 2 );

		//! グラフィックス･オブジェクトを解放する.
		g2d.dispose();
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	スクロールバーを操作したときのイベント処理.
	 *
	 *	@param	e	イベント･パラメータ.
	 */
	public void adjustmentValueChanged( AdjustmentEvent e ) {

//		JScrollBar sb = (JScrollBar)e.getSource();		// not used.

		//! バイナリ表示ビューを再描画する.
		repaint();
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	マウスホイールを操作したときのイベント処理.
	 *
	 *	@param	e	イベント･パラメータ.
	 */
	private void onMouseWheel( MouseWheelEvent e ){

//		int x   = e.getX();				// not used.
//		int y   = e.getY();				// not used.
		int val = scrollV.getValue();

		if( e.getWheelRotation() > 0 ){
			val += e.getWheelRotation();
			if( val > scrollV.getMaximum() ){
				val = scrollV.getMaximum();
			}
			scrollV.setValue( val );
		}else
		if( e.getWheelRotation() < 0 ){
			val += e.getWheelRotation();
			if( val < scrollV.getMinimum() ){
				val = scrollV.getMinimum();
			}
			scrollV.setValue( val );
		}
		//! バイナリ表示ビューを再描画する.
		repaint();
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ウインドウサイズを変更したときのイベント処理.
	 *
	 *	@param	e	イベント･パラメータ.
	 */
	private void onResized( ComponentEvent e ) {

		//! スクロールバーを設定する.
		setScrollBar();
	}
}

/* End of file */
