/* -----------------------------------------------------------------------
 *	Copylight(C) 2018 Kanazawa-soft-design,LLC.
 * ----------------------------------------------------------------------- */
/*!
 *	@file	FileListView.java
 *
 *	@brief	ファイル一覧表示ウインドウ (Java + Swing).
 *
 *	@author	金澤 宣明
 */
package sample.JavaSwing001;

import java.io.File;
import java.sql.Date;
import java.text.SimpleDateFormat;
import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.filechooser.FileSystemView;
import javax.swing.table.DefaultTableCellRenderer;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Point;

@SuppressWarnings("serial")
public class FileListView extends JTable {

	private String					currentDir = "";	//!< 一覧表示するディレクトリのパス.
	private FileViewEventListener	notifyEvent;		//!< イベント通知先.

	/* ----------------------------------------------------------------------- */
	/**
	 *	コンストラクタ.
	 *
	 */
	FileListView() {

		DefaultTableModel model = new DefaultTableModel();
		setModel( model );

		//! カラム名を登録する.
		model.setColumnIdentifiers( new String[] { "ファイル名", "サイズ", "更新日時" } );

		//! それぞれのカラムにレンダラーを設定する.
		getColumnModel().getColumn(0).setCellRenderer( new FileNameTableCellRenderer() );
		getColumnModel().getColumn(1).setCellRenderer( new FileSizeTableCellRenderer() );
		getColumnModel().getColumn(2).setCellRenderer( new FileDateTableCellRenderer() );

		//! 罫線を非表示にする.
		setShowHorizontalLines( false );
		setShowVerticalLines(   false );

		//! テーブルの項目をダブルクリックしたときのイベント処理を登録する.
		addMouseListener( new MouseAdapter() {
			@Override
			public void mouseClicked( MouseEvent me ) {
				if( me.getButton() == MouseEvent.BUTTON1 && me.getClickCount() == 2 ){
					onDblClick( me );
				}
			}
		});

		//! ファイル一覧にドライブ名を表示する.
		showDriveName( model );

		setVisible( true );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	リストビューのセルが編集可能かどうかの問い合わせ.
	 *
	 *	@param		row		問い合わせるセルの行番号.
	 *	@param		column	問い合わせるセルのカラム番号.
	 *
	 *	@return		常にfalseを返す.
	 */
	@Override
	public boolean isCellEditable( int row, int column ) {

		//! リストビューのセルの編集を不可にする.
		return false;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル一覧を更新する.
	 *
	 *	@param		path	一覧表示するディレクトリ名.
	 *
	 */
	public void updateFileList( String path ) {

		//! 一覧表示するディレクトリ名を保存する.
		currentDir = path;

		DefaultTableModel model = (DefaultTableModel)getModel();

		if( path.length() > 0 ){
			//! 指定されたパスのファイル一覧を生成する.
			showFileName( model, path );
		}else{
			//! パスが指定されていない場合はドライブ一覧を生成する.
			showDriveName( model );
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル一覧にドライブ名を表示する.
	 *
	 *	@param	model	リストビューのテーブル･モデル.
	 *
	 */
	private void showDriveName( DefaultTableModel model ) {

		//! ファイル一覧の表示を一旦クリアする.
		model.setRowCount( 0 );

		//! 論理ドライブ名をすべて取得する.
		File[] roots = File.listRoots();

		for( File drv : roots ){
			//! ドライブ名をファイル一覧に追加する.
			model.addRow( new File[] { drv, drv, drv } );
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル一覧にファイル名の一覧を表示する.
	 *
	 *	@param	model	リストビューのテーブル･モデル.
	 *	@param	path	ファイル一覧を表示するディレクトリの絶対パス.
	 *
	 */
	private void showFileName( DefaultTableModel model, String path ) {

		//! ファイル一覧の表示を一旦クリアする.
		model.setRowCount( 0 );

		//! 一つ上のディレクトリに移動する為の「..」をリストの先頭に追加する.
		model.addRow( new String[] { "..", "<DIR>", "" } );

		//! 指定されたディレクトリ直下のファイルとディレクトリのリストを取得する.
		path += "\\";
		File   dir   = new File( path );
		File[] files = dir.listFiles();
		if( files == null ){
			return;
		}

		//! 先にサブディレクトリ名をリストに追加する.
		for( int i = 0; i < files.length; i++ ){
			File file = files[i];

			//! 隠しファイルまたは読み込み不可のファイルは表示しない.
			if( file.isHidden() || !file.canRead() ){
				continue;
			}
			//! 先にディレクトリを表示するのでファイルの場合は表示しない.
			if( !file.isDirectory() ){
				continue;
			}
			//! サブディレクトリをファイル一覧に追加する.
			model.addRow( new File[] { file, file, file } );
		}

		//! 次にファイル名をリストに追加する.
		for( int i = 0; i < files.length; i++ ){
			File file = files[i];

			//! 隠しファイルまたは読み込み不可のファイルは表示しない.
			if( file.isHidden() || !file.canRead() ){
				continue;
			}
			//! ディレクトリは既に表示済みなので表示しない.
			if( file.isDirectory() ){
				continue;
			}
			//! ファイル名をファイル一覧に追加する.
			model.addRow( new File[] { file, file, file } );
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	イベント通知先を登録する.
	 *
	 *	@param	listener	イベント･リスナー.
	 */
	public void addEventListener( FileViewEventListener listener ) {
		notifyEvent = listener;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル一覧の項目がダブルクリックされたときの操作イベント.
	 *
	 * @param	me		マウス･イベントのイベント･パラメータ.
	 *
	 */
	private void onDblClick( MouseEvent me ) {

		//! リストビューから選択された項目番号を取り出す.
		Point pt = me.getPoint();
		int idx = rowAtPoint( pt );
		if( idx >= 0 ){
			int row = convertRowIndexToModel( idx );

			DefaultTableModel model = (DefaultTableModel)getModel();

			Object value = model.getValueAt( row, 0 );
			if( value instanceof File ){
				String filename = value.toString();
				if( filename.length() > 0 ){
					File file = new File( filename );
					if( file.isFile() ){
						//! ファイルの場合はバイナリ表示する.
						notifyEvent.showBinaryFile( filename );
					}else{
						//! ディレクトリの場合は選択されたディレクトリに移動する.
						notifyEvent.expandDirectory( filename );
					}
				}
			}else
			if( value instanceof String ){
				String name = (String)value;

				if( name.equals( ".." ) ){
					//! ".."の場合、カレントパスの最後のディレクトリ名を取り除く.
					String path = "";
					int    pos  = currentDir.lastIndexOf( '\\' );
					if( pos >= 0 ){
						path = currentDir.substring( 0, pos );
					}
					notifyEvent.expandDirectory( path );
				}
			}
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル名のセルレンダラー.
	 *
	 */
	class FileNameTableCellRenderer extends DefaultTableCellRenderer {

		FileSystemView fs = FileSystemView.getFileSystemView();

		@Override
		public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column ){

			super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );

			String str = "";
			if( value instanceof File ){
				File file = (File)value;
				if( fs.isDrive( file ) ){
					//! ドライブの場合パスからドライブレターを取り出す.
					String path = file.toString();
					int    pos  = path.lastIndexOf( '\\' );
					str = path.substring( 0, pos );
				}else{
					//! ファイル名をリストに追加する.
					str = fs.getSystemDisplayName( file );
				}
			}else
			if( value instanceof String ){
				//! 文字列の場合はそのままリストに追加する.
				str = (String)value;
			}
			//! リストにファイル名を追加する.
			setText( str );
			return this;
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイルサイズのセルレンダラー.
	 *
	 */
	class FileSizeTableCellRenderer extends DefaultTableCellRenderer {

		FileSystemView fs = FileSystemView.getFileSystemView();

		@Override
		public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column ){

			super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );

			String str = "";
			if( value instanceof File ){
				File file = (File)value;
				if( fs.isDrive( file ) ){
					//! ドライブの場合は""(空白)をリストに追加する.
				}else
				if( file.isDirectory() ){
					//! ディレクトリの場合は「<DIR>」をリストに追加する.
					str = "<DIR>";
				}else{
					//!ファイルの場合ファイルサイズをカンマ区切りで編集してリストに追加する.
					str = String.format( "%1$,3d ", file.length() );
				}
			}else
			if( value instanceof String ){
				//! 文字列の場合はそのままリストに追加する.
				str = (String)value;
			}
			//! リストにファイルサイズを追加する.
			setText( str );

			//! 文字列を右寄せにする.
			setHorizontalAlignment( RIGHT );
			return this;
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	更新日付のセルレンダラー.
	 *
	 */
	class FileDateTableCellRenderer extends DefaultTableCellRenderer {

		@Override
		public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column ){

			super.getTableCellRendererComponent( table, value, isSelected, hasFocus, row, column );

			String str = "";
			if( value instanceof File ){
				File file = (File)value;

				if( file.isFile() ){
					//! ファイルの場合は更新日付を文字列にする.
					SimpleDateFormat sdf = new SimpleDateFormat( "yyyy/MM/dd hh:mm" );
					str = sdf.format( new Date( file.lastModified() ) );
				}
			}else
			if( value instanceof String ){
				//! 文字列の場合はそのままリストに追加する.
				str = (String)value;
			}
			//! リストにファイルサイズを追加する.
			setText( str );

			return this;
		}
	}
}

/* End of file */
