/* -----------------------------------------------------------------------
 *	Copylight(C) 2018 Kanazawa-soft-design,LLC.
 * ----------------------------------------------------------------------- */
/*!
 *	@file	FileTreeView.java
 *
 *	@brief	ディレクトリ･ツリー表示ウインドウ (Java + Swing).
 *
 *	@author	金澤 宣明
 */
package sample.JavaSwing001;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeWillExpandListener;
import java.io.File;

@SuppressWarnings("serial")
public class FileTreeView extends JTree {

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリ･ツリー･アイテム.
	 *
	 */
	private class TreeItem {

		private String itemName;					//!< 名前.

		TreeItem( String name ){
			itemName = name;
		}
		public String toString() {
			return itemName;
		}
	}

	private FileViewEventListener notifyEvent;		//!< イベント通知先.

	/* ----------------------------------------------------------------------- */
	/**
	 *	コンストラクタ.
	 *
	 */
	FileTreeView() {

		//! コンピュータ名をディレクトリツリーのルートに登録する.
		DefaultMutableTreeNode rootNode = createRootDirectory();

		//! ディレクトリツリーにドライブ名を登録する.
		addDriveName( rootNode );

		//! ディレクトリツリーのディレクトリが展開されるときのイベント処理を登録する.
		addTreeWillExpandListener( new TreeWillExpandListener() {
			@Override
			public void treeWillCollapse( TreeExpansionEvent e ) throws ExpandVetoException {
			}

			@Override
			public void treeWillExpand( TreeExpansionEvent e) throws ExpandVetoException {
				fileTreeExpand( e );
			}
		});

		//! ディレクトリの選択が変更されるときのイベント処理を登録する.
		addTreeSelectionListener( new TreeSelectionListener(){
			@Override
			public void valueChanged( TreeSelectionEvent e ){
				fileTreeSelectChanged( e );
			}
		});

		//! ルートノードを展開表示する.
		expandRow( 0 );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	イベント通知先を登録する.
	 *
	 *	@param	listener	イベント･リスナー.
	 */
	public void addEventListener( FileViewEventListener listener ) {
		notifyEvent = listener;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリツリーのディレクトリが展開されるときのイベント処理.
	 *
	 *	@param	e	イベントパラメータ.
	 */
	private void fileTreeExpand( TreeExpansionEvent e ) {

		//! 展開されるノードを取得する.
		TreePath path = e.getPath();
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
		if( node == null ){
			return;
		}
		//! 展開されるディレクトリのサブディレクトリを一旦クリアする.
		node.removeAllChildren();

		//! 展開されるディレクトリのフルパスを取得する.
		String filePath = getAbsolutePath( path );

		if( filePath.length() > 0 ){
			//! 展開されるディレクトリのサブディレクトリを表示する.
			addDirectory( node, filePath, false );
		}else{
			//! フルパス名が無い場合はルートの展開なのでドライブ名を表示する.
			addDriveName( node );
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリの選択が変更されるときのイベント処理.
	 *
	 *	@param	e	イベントパラメータ.
	 */
	private void fileTreeSelectChanged( TreeSelectionEvent e ) {

		//! 選択されたディレクトリのフルパスを取得する.
		TreePath path     = e.getPath();
		String   filePath = getAbsolutePath( path );

		//! ファイル一覧の表示を更新する.
		notifyEvent.updateFileList( filePath );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	コンピュータ名をディレクトリツリーのルートに登録する.
	 *
	 *	@return	生成したルートディレクトリのノード.
	 */
	private DefaultMutableTreeNode createRootDirectory() {

		//! コンピュータ名を取得する.
		String name = "My Computer";
		try {
			name = java.net.InetAddress.getLocalHost().getHostName();
		}catch( Exception e ){
		}
		//! 取得したコンピュータ名でノードを生成する.
		TreeItem rootItem = new TreeItem( name );
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode( rootItem );

		//! ディレクトリツリーに生成したルートノードを登録する.
		DefaultTreeModel model = new DefaultTreeModel( rootNode );
		setModel( model );

		return rootNode;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリツリーにドライブ名を登録する.
	 *
	 *	@param	root	ディレクトリツリーのROOTノード.
	 */
	private void addDriveName( DefaultMutableTreeNode root ) {

		//! 論理ドライブ名をすべて取得する.
		File[] roots = File.listRoots();

		for( File dir : roots ){
			//! ドライブ文字列からバックスラッシュ('\')を取り除く.
			String path = dir.getPath();
			String name = path.substring( 0, path.length() - 1 );

			//! ドライブ名をROOTノードに追加する.
			DefaultMutableTreeNode node = addTreeNode( root, name );

			//! サブディレクトリが存在するかどうかを調べる.
			addDirectory( node, dir.getPath(), true );
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリツリーにディレクトリ名を登録する.
	 *
	 *	@param	parent	ディレクトリ名を登録するノード.
	 *	@param	path	該当ノードに対応した絶対パスのディレクトリ名.
	 *	@param	isDummy	true = サブディレクトリが存在するかどうかだけ調べる.
	 */
	private void addDirectory( DefaultMutableTreeNode parent, String path, boolean isDummy ) {

		//! ディレクトリ直下のファイル一覧を取得する.
		path += "\\";
		File dir     = new File( path );
		File[] files = dir.listFiles();

		if( files == null ){
			return;
		}
		for( int i = 0; i < files.length; i++ ){
			File file = files[i];
			if( file.isDirectory() && !file.isHidden() && file.canRead() ){
				//! サブディレクトリを追加する.
				DefaultMutableTreeNode node = addTreeNode( parent, file.getName() );

				//! サブディレクトリの存在チェックの場合は、1件検出した時点で処理を終了する.
				if( isDummy ){
					return;
				}
				//! サブディレクトリが存在するかどうかを調べる.
				addDirectory( node, file.getPath(), true );
			}
		}
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリ名またはドライブ名をディレクトリツリーに追加する.
	 *
	 *	@param	parent	ディレクトリを登録するノード.
	 *	@param	name	ディレクトリ名.
	 *
	 *	@return	生成したサブディレクトリのノード.
	 */
	private DefaultMutableTreeNode addTreeNode( DefaultMutableTreeNode parent, String name ) {

		//! ディレクトリ名のノードを生成する.
		TreeItem				item = new TreeItem( name );
		DefaultMutableTreeNode	node = new DefaultMutableTreeNode( item );

		//! 親ノードに作成したノードを追加する.
		parent.add( node );

		return node;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	指定されたノードに対応した絶対パスを取得する.
	 *
	 *	@param	path	絶対パスを取得するノードのツリーパス.
	 *
	 *	@return	指定されたノードに対応した絶対パス.
	 */
	private String getAbsolutePath( TreePath path ) {

		String filePath = "";

		TreePath parent = path.getParentPath();
		if( parent != null ){
			//! 親ディレクトリの絶対パスを取得する.
			filePath += getAbsolutePath( parent );

			//! 親ディレクトリがある場合は連結する為にセパレータ("\")を付ける.
			if( filePath.length() > 0 ){
				filePath += "\\";
			}
			//! 親ディレクトリとファイル名を連結して絶対パス名にする.
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
			filePath += node.toString();
		}else{
			;
		}
		return filePath;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	指定されたディレクトリまでのノードを展開する.
	 *
	 *	@param	path	ディレクトリの絶対パス名.
	 */
	public void expandAndSelectDirectory( String path ) {

		TreeModel model = getModel();

		DefaultMutableTreeNode node = (DefaultMutableTreeNode)model.getRoot();

		//! indexOf()検索で最後まで検索するようにパスの最後に検索文字を付加する.
		String wkPath = path + "\\";

		int next;
		int pos = 0;
		while( node != null && (next = wkPath.indexOf( '\\', pos )) != -1 ){

			//! 子ノードが無い場合は処理を終了する.
			if( model.getChildCount( node ) == 0 ){
				node = null;
				break;
			}
			//! "\"が連続している場合は読み飛ばす.
			if( next == pos ){
				pos = next + 1;
				continue;
			}
			//! ディレクトリ名を絶対パスから取り出す.
			String dirName = wkPath.substring( pos, next );
			pos = next + 1;

			for( int i = 0; i < model.getChildCount( node ); i++ ){

				DefaultMutableTreeNode child = (DefaultMutableTreeNode)model.getChild( node, i );

				//! 同じディレクトリ名のノードを見つけたらループを抜ける.
				if( dirName.compareToIgnoreCase( child.toString() ) == 0 ){
					node = child;
					break;
				}
			}
		}
		if( node != null ){
			TreePath treePath = new TreePath( node.getPath() );

			//! 選択されたノードまでを展開する.
			expandPath( treePath );

			//! ノードを選択状態にする.
			setSelectionPath( treePath );
		}
	}
}

/* End of file */
