/* -----------------------------------------------------------------------
 *	Copylight(C) 2018 Kanazawa-soft-design,LLC.
 * ----------------------------------------------------------------------- */
/*!
 *	@file	JavaSwing001.java
 *
 *	@brief	バイナリ形式でファイルの内容を表示するサンプル･プログラム.
 *
 *	@author	金澤 宣明
 */
package sample.JavaSwing001;

import javax.swing.*;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

@SuppressWarnings("serial")
public class JavaSwing001 extends JFrame implements FileViewEventListener {

	private FileTreeView	fileTreeView;		//!< ディレクトリ･ツリー表示ビュー.
	private FileListView	fileListView;		//!< ファイル一覧表示ビュー.
	private FileDataView	fileDataView;		//!< バイナリデータ表示ビュー.
	private JLabel			statusBar;			//!< ステータスバー.

	/* ----------------------------------------------------------------------- */
	/**
	 *	コンストラクタ.
	 *
	 *	@param	title	メインウインドウのタイトルバーに表示する文字列.
	 */
	JavaSwing001( String title ) {

		//! タイトルバーを表示する.
		setTitle( title );

		//! ウインドウのサイズと表示位置を設定する.
		setBounds( 200, 200, 800, 600 );

		//!「X」ボタンのイベントを設定する.
		setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );

		//! ネイティブシステムの Look & Feelをセット.
		try{
			UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
		} catch( Exception e ){
			e.printStackTrace();
		}

		//! メニューを作成する.
		createMenu();

		//! ディレクトリ･ツリー表示ビューを生成する.
		JScrollPane treePanel = createTreeView();

		//! ファイル一覧表示ビューを生成する.
		JScrollPane listPanel = createListView();

		//! バイナリデータ表示ビューを生成する.
		JPanel dataPanel = createDataView();

		//! 右側の分割ウインドウを作成する.
		JSplitPane rightPanel = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		rightPanel.setLeftComponent(  listPanel );
		rightPanel.setRightComponent( dataPanel );

		//! 全体の分割ウインドウを作成する.
		JSplitPane mainPanel = new JSplitPane();
		mainPanel.setLeftComponent(  treePanel  );
		mainPanel.setRightComponent( rightPanel );

		Container contentPane = getContentPane();
		contentPane.add( mainPanel, BorderLayout.CENTER );

		//! ステータスバーを生成する.
		contentPane.add( createStatusBar(), BorderLayout.PAGE_END );

		//! イベント通知先をセットする.
		fileTreeView.addEventListener( this );
		fileListView.addEventListener( this );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ディレクトリ･ツリー表示ビューを生成する.
	 *
	 *	@return		生成したディレクトリ･ツリー表示ビュー.
	 */
	private JScrollPane createTreeView() {

		//! スクロール・パネルを生成する.
		JScrollPane panel = new JScrollPane();

		//! ツリービューを生成する.
		fileTreeView = new FileTreeView();

		//! スクロールビューにツリービューを配置する.
		panel.setViewportView( fileTreeView );

		panel.setPreferredSize( new Dimension(150, 400) );

		return panel;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル一覧表示ビューを生成する.
	 *
	 *	@return		生成したファイル一覧表示ビュー.
	 */
	private JScrollPane createListView() {

		//! スクロール・パネルを生成する.
		JScrollPane panel = new JScrollPane();

		//! リストビューを生成する.
		fileListView = new FileListView();

		//! スクロールビューにリストビューを配置する.
		panel.setViewportView( fileListView );

		//! リストビューの背景色を白色にする.
		panel.getViewport().setOpaque( true );
		panel.getViewport().setBackground( Color.WHITE );

		panel.setPreferredSize( new Dimension(400, 100) );

		return panel;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	バイナリデータ表示ビューを生成する.
	 *
	 *	@return		生成したバイナリデータ表示ビュー.
	 */
	private JPanel createDataView() {

		//! バイナリデータ表示ビューの土台になるパネルを生成する.
		JPanel panel = new JPanel();
		panel.setLayout( new BorderLayout() );

		//! 水平スクロールバーをパネルに配置する.
		JScrollBar sbH = new JScrollBar( JScrollBar.HORIZONTAL );
		panel.add( sbH, BorderLayout.SOUTH );

		//! 垂直スクロールバーをパネルに配置する.
		JScrollBar sbV = new JScrollBar( JScrollBar.VERTICAL );
		panel.add( sbV, BorderLayout.EAST );

		//! バイナリデータ表示ビューをパネルに配置する.
		fileDataView = new FileDataView( sbV, sbH );
		panel.add( fileDataView, BorderLayout.CENTER );

		return panel;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ステータスバーを生成する.
	 *
	 *	@return		生成したステータスバー.
	 */
	private JPanel createStatusBar() {

		//! ステータスバーになるパネルを生成する.
		JPanel bar = new JPanel( new FlowLayout( FlowLayout.LEFT ) );

		//! 窪みの影を付ける.
//		bar.setBorder( new BevelBorder( BevelBorder.LOWERED ) );

		statusBar = new JLabel( "" );

		//! パネルにラベルを配置する.
		bar.add( statusBar );

		return bar;
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	メニューを作成する.
	 *
	 */
	private void createMenu() {

		//! メニューバーを生成する.
		JMenuBar menubar = new JMenuBar();

		//! メニューバーに「ファイル」を追加.
		JMenu menu1 = new JMenu( "ファイル" );
		menubar.add( menu1 );

		//!「ファイル」にサブメニュー「アプリケーションの終了」を追加.
		JMenuItem menuitem1_1 = new JMenuItem( "アプリケーションの終了" );
		menu1.add( menuitem1_1 );
		menuitem1_1.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent e ) {
				menuAppExit( e );
			}
		});

		//! メニューバーに「ヘルプ」を追加.
		JMenu menu2 = new JMenu( "ヘルプ" );
		menubar.add( menu2 );

		//!「ヘルプ」にサブメニュー「バージョン情報」を追加.
		JMenuItem menuitem2_1 = new JMenuItem( "バージョン情報" );
		menu2.add( menuitem2_1 );
		menuitem2_1.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent e ) {
				menuAppAbout( e );
			}
		});

		//! フレームウインドウに作成したメニューを登録する.
		setJMenuBar( menubar );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	メニューイベント : 「アプリケーションの終了」.
	 *
	 */
	private void menuAppExit( ActionEvent e ) {

		//! ウインドウをクローズする.
		setVisible( false );
		dispose();

		//! アプリケーションを終了する.
		System.exit( 0 );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	メニューイベント : 「バージョン情報」.
	 *
	 */
	private void menuAppAbout( ActionEvent e ) {

		//! バージョン情報ダイアログを生成する.
		AboutDialog dlg = new AboutDialog( this, true );

		//! バージョン情報ダイアログを表示する.
		dlg.setVisible( true );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイル一覧を更新する.
	 *
	 *	@param	path	一覧表示するディレクトリの絶対パス.
	 */
	public void updateFileList( String path ) {

		//! ファイル一覧を更新する.
		fileListView.updateFileList( path );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	ファイルの内容をバイナリ形式で表示する.
	 *
	 *	@param	path	バイナリ表示するファイルの絶対パス.
	 */
	public void showBinaryFile( String path ) {

		//! バイナリ表示ウインドウに表示するファイル名を通知する.
		fileDataView.showBinaryFile( path );

		//! ステータスバーにファイル名を表示する.
		statusBar.setText( path );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	指定したディレクトリを展開表示する.
	 *
	 *	@param	path	展開するディレクトリの絶対パス.
	 */
	public void expandDirectory( String path ) {

		//! ディレクトリツリーの該当ディレクトリを展開する.
		fileTreeView.expandAndSelectDirectory( path );

		//! ファイル一覧を更新する.
		fileListView.updateFileList( path );
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	アプリケーションのエントリ関数.
	 *
	 *	@param	args	コマンドライン引数.
	 */
	public static void main( String[] args ) {
		/*!
		 * Swingアプリケーションは専用スレッドで実行することを想定しているので
		 * メインの実行スレッドから実行してはいけないらしいです.
		 */
		AppStarter starter = new AppStarter( args );
		SwingUtilities.invokeLater( starter );
	}
}


/* =========================================================================== */
/**
 *	Swingアプリケーションの実行スレッド.
 *
 */
class AppStarter extends Thread
{
//	private String[] args;		// not used.

	/* ----------------------------------------------------------------------- */
	/**
	 *	コンストラクタ.
	 *
	 *	@param	args	コマンドライン引数.
	 */
	public AppStarter( String[] args ){
//		this.args = args;		// not used.
	}

	/* ----------------------------------------------------------------------- */
	/**
	 *	スレッドの実行処理.
	 *
	 */
	public void run()
	{
		JavaSwing001 frame = new JavaSwing001( "バイナリ形式でファイルの内容を表示 (Java Swing)" );

		frame.setVisible( true );
	}
}

/* End of file */
